import {
  select,
  csv,
  scaleLinear,
  scaleSqrt,
  extent,
  max,
  axisLeft,
  axisBottom
} from 'd3';

export const scatterplot = (selection, props) => {
  const {
    columns,
    changeColumn,
    tidyStr,
    yValue,
    xValue,
    zValue,
    margin,
    width,
    height,
    data,
    title
  } = props;
  
  const innerWidth = width - margin.left - margin.right;
  const innerHeight = height - margin.top - margin.bottom;
   
  const axisFormat = (number, attribute) => 
  	d3.format(attribute)(number)
  		.replace('G', 'B')
  		.replace('500m', '0.5')
  
  const yAxisFormat = number => axisFormat(number, '.2s')
  const xAxisFormat = number => axisFormat(number, '.2s')
  
  const xScale = scaleLinear()
    .domain([0, max(data, xValue)])
    .range([0, innerWidth])
    .nice();
  
  const yScale = scaleLinear()
    .domain([0, max(data, yValue)])
    .range([innerHeight, 0])
    .nice();
  
	const zScale = scaleSqrt()
    .domain([0, max(data, zValue)])
    .range([0, 25]);
  
  
  const g = selection.selectAll('.container').data([null]);
  const gEnter = g
    .enter().append('g')
      .attr('class', 'container');
  gEnter
    .merge(g)
      .attr('transform',
        `translate(${margin.left},${margin.top})`
      );
  
  // Append the title  
  gEnter.append('text')
    .attr('class', 'title')
    .attr('y', -30)
    .text(title)
  
  const xAxis = axisBottom(xScale)
    .tickSize(-innerHeight)
    .tickPadding(15)
  	.tickFormat(xAxisFormat)
  
  const yAxis = axisLeft(yScale)
    .tickSize(-innerWidth)
    .tickPadding(10)
    .tickFormat(yAxisFormat)
  
  const yAxisG = g.select('.y-axis');
  const yAxisGEnter = gEnter
    .append('g')
      .attr('class', 'y-axis');
  yAxisG
    .merge(yAxisGEnter)
  		.transition().duration(500)
      .call(yAxis)
  
  const yAxisLabelText = yAxisGEnter
    .append('text')
      .attr('class', 'axis-label')
      .attr('y', -75)
      .attr('fill', 'black')
      .attr('transform', `rotate(-90)`)
      .attr('text-anchor', 'middle')
    .merge(yAxisG.select('.axis-label'))
      .attr('x', -innerHeight / 2)
      .text(tidyStr(columns.y))
  
  const xAxisG = g.select('.x-axis')
  const xAxisGEnter = gEnter
    .append('g')
      .attr('class', 'x-axis')
  xAxisG
    .merge(xAxisGEnter)
      .attr('transform', `translate(0,${innerHeight})`)
  		.transition().duration(500)
      .call(xAxis)
  
  const xAxisLabelText = xAxisGEnter
    .append('text')
      .attr('class', 'axis-label')
      .attr('y', 64)
      .attr('fill', 'black')
    .merge(xAxisG.select('.axis-label'))
      .attr('x', innerWidth / 2)
      .text(tidyStr(columns.x))

  // Handle circles
  const circles = g.merge(gEnter)
    .selectAll('.plot-circle').data(data, d => d.name)
  const circlesEnter = circles
    .enter().append('circle')
  circlesEnter.attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', 0)
  		.attr('class', 'plot-circle')
  .merge(circles)
  	.transition().duration(500)
    .delay((d, i) => i * 5)
      .attr('cy', d => yScale(yValue(d)))
      .attr('cx', d => xScale(xValue(d)))
      .attr('r', d => zScale(zValue(d)))
  circlesEnter
      .append("svg:title")
   			.text((d) => d.name)
  circles.exit()
    .transition().duration(500)
    	.attr('r', 0)
    	.remove();

  

  
  // Creating steps for legend
  const ticks = zScale.ticks(5).filter(d => d !== 0).reverse()
  
  const sizeLegendGroup = g.select('.size-legend')
  const sizeLegendEnter = gEnter.append('g')
  	.attr('class', 'size-legend')
    .attr('transform', 'translate(' + (innerWidth + 50) + ', 0)')
  sizeLegendEnter.append('text')
    .attr('class', 'legend-header')
  	.merge(sizeLegendGroup.select('.legend-header'))
  	.text(tidyStr(columns.z))
  sizeLegendGroup.selectAll('.g-size-legend').remove()
  const ticksEnter = sizeLegendEnter
    .merge(sizeLegendGroup)
      .selectAll('.g-size-legend').data(ticks)
  		.enter().append('g')
			.attr('class', 'g-size-legend')
  			.attr('transform', (d, i) => 'translate(0,' + (i*35 + 30) + ')')
  			.merge(sizeLegendGroup.selectAll('.g-size-legend'))
  ticksEnter.append('circle')
    .attr('class', 'legend-circle')
    .attr('r', d => {
    	return zScale(d)
		})
  ticksEnter.append('text')
  	.text(d => d)
  	.attr('dy', '0.32em')
  	.attr('x', d => zScale(d) + 6)

}
